﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dlubal.RFEM5;

namespace Dlubal.Examples.MemberVaryingLoads
{
  public partial class FormMain : Form
  {
    public FormMain()
    {
      InitializeComponent();
    }

    private void CreateModel(IModel model)
    {
      // Gets interface to model data.
      IModelData data = model.GetModelData();

      // Creates nodes
      Node[] nodes = new Node[2];
      nodes[0].No = 1;
      nodes[0].X = 0.0;
      nodes[0].Y = 0.0;
      nodes[0].Z = 0.0;
      nodes[1].No = 2;
      nodes[1].X = 3.0;
      nodes[1].Y = 0.0;
      nodes[1].Z = 0.0;

      // Creates line
      Line line = new Line();
      line.No = 1;
      line.NodeList = "1,2";
      line.Type = LineType.PolylineType;

      // Creates material
      Material material = new Material();
      material.No = 1;
      material.Description = "Concrete C30/37";
      material.TextID = "Beton C30/37";

      // Creates cross-section
      CrossSection crsc = new CrossSection();
      crsc.No = 1;
      crsc.Description = "Rectangle 100/200";
      crsc.TextID = "RECHTECK 100/200";
      crsc.MaterialNo = 1;

      // Creates member
      Member member = new Member();
      member.No = 1;
      member.Type = MemberType.Beam;
      member.LineNo = 1;
      member.StartCrossSectionNo = 1;
      member.EndCrossSectionNo = 1;

      // Write model objects to RFEM
      data.PrepareModification();
      try
      {
        data.SetNodes(nodes);
        data.SetLine(line);
        data.SetMaterial(material);
        data.SetCrossSection(crsc);
        data.SetMember(member);
      }
      finally
      {
        data.FinishModification();
      }
    }
    private void CreateLoad(IModel model)
    {
      // Gets interface to loads.
      ILoads loads = model.GetLoads();

      // Creates load case
      LoadCase lcData = new LoadCase();
      lcData.Loading.No = 1;
      lcData.ActionCategory = ActionCategoryType.Permanent;
      lcData.Description = "New Load Case";
      //lcData.Analysis.LoadingFactor = 1.0;

      // Writes load case to RFEM
      ILoadCase lc = null;
      loads.PrepareModification();
      try
      {
        lc = loads.SetLoadCase(lcData);
      }
      finally
      {
        loads.FinishModification();
      }

      // Creates member varying load
      MemberLoad load = new MemberLoad();
      load.No = 1;
      load.ObjectList = "1"; // Member No.1
      load.Type = LoadType.ForceType;
      load.Direction = LoadDirectionType.GlobalZType;
      load.Distribution = LoadDistributionType.VaryingType;
      load.RelativeDistances = false;
      load.LoadArray = new double[10, 2];
      load.LoadArray.SetValue(0.0, 0, 0);
      load.LoadArray.SetValue(10.0, 1, 0);
      load.LoadArray.SetValue(20.0, 2, 0);
      load.LoadArray.SetValue(30.0, 3, 0);
      load.LoadArray.SetValue(40.0, 4, 0);
      load.LoadArray.SetValue(50.0, 5, 0);
      load.LoadArray.SetValue(60.0, 5, 0);
      load.LoadArray.SetValue(70.0, 6, 0);
      load.LoadArray.SetValue(80.0, 7, 0);
      load.LoadArray.SetValue(90.0, 8, 0);
      load.LoadArray.SetValue(100.0, 9, 0);
      load.LoadArray.SetValue(0.0, 0, 1);
      load.LoadArray.SetValue(1000.0, 1, 1);
      load.LoadArray.SetValue(3000.0, 2, 1);
      load.LoadArray.SetValue(2000.0, 3, 1);
      load.LoadArray.SetValue(2000.0, 4, 1);
      load.LoadArray.SetValue(5000.0, 5, 1);
      load.LoadArray.SetValue(4000.0, 6, 1);
      load.LoadArray.SetValue(3000.0, 7, 1);
      load.LoadArray.SetValue(5000.0, 8, 1);
      load.LoadArray.SetValue(0.0, 9, 1);

      // Write member load to RFEM.
      lc.PrepareModification();
      try
      {
        lc.SetMemberLoad(load);
      }
      finally
      {
        lc.FinishModification();
        lc = null;
        loads = null;
      }
    }

    private void btnSetMemberLoad_Click(object sender, EventArgs e)
    {
      IApplication app = null;
      IModel model = null;

      try
      {
        // Gets interface to running RFEM application.
        app = Marshal.GetActiveObject("RFEM5.Application") as IApplication;
        // Locks RFEM license
        app.LockLicense();

        // Gets interface to active RFEM model.
        model = app.GetActiveModel();

        CreateModel(model);
        CreateLoad(model);
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        // Releases interface to RFEM model.
        model = null;

        // Unlocks license and releases interface to RFEM application.
        if (app != null)
        {
          app.UnlockLicense();
          app = null;
        }

        // Cleans Garbage Collector and releases all cached COM interfaces.
        System.GC.Collect();
        System.GC.WaitForPendingFinalizers();
      }
    }
  }
}
