﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dlubal.RFEM5;

namespace Dlubal.Examples.NonlinearHingesExample
{
  public partial class MainForm : Form
  {
    public MainForm()
    {
      InitializeComponent();
    }

    private void btnSetNonlinearHinges_Click(object sender, EventArgs e)
    {
      IApplication app = null;
      IModel model = null;

      try
      {
        // Gets interface to running RFEM application.
        app = Marshal.GetActiveObject("RFEM5.Application") as IApplication;
        // Locks RFEM license
        app.LockLicense();

        // Gets interface to active RFEM model.
        model = app.GetActiveModel();

        // Gets interface to model data.
        IModelData data = model.GetModelData();

        // Creates new nodes.
        Node[] nodes = new Node[2];
        nodes[0].No = 1;
        nodes[0].X = 1.0;
        nodes[0].Y = 1.0;
        nodes[0].Z = 0.0;
        nodes[1].No = 2;
        nodes[1].X = 3.0;
        nodes[1].Y = 1.0;
        nodes[1].Z = 0.0;

        // Creates new line.
        Line line = new Line();
        line.No = 1;
        line.NodeList = "1,2";
        line.Type = LineType.PolylineType;

        // Creates new cross-section
        CrossSection crsc = new CrossSection();
        crsc.No = 1;
        crsc.Description = "IPE 300";
        crsc.TextID = "IPE 300";
        crsc.MaterialNo = 1;

        // Creates new hinge.
        MemberHinge hinge = new MemberHinge();
        hinge.No = 1;
        hinge.ReferenceSystem = ReferenceSystemType.LocalSystemType;
        hinge.TranslationalNonlinearityX = NonlinearityType.WorkingDiagramType;
        hinge.TranslationalNonlinearityZ = NonlinearityType.WorkingDiagramType;

        // Creates new member.
        Member member = new Member();
        member.No = 1;
        member.LineNo = 1;
        member.Type = MemberType.Beam;
        member.StartCrossSectionNo = 1;
        member.StartHingeNo = 1;

        // Creates new working diagram for the support - X.
        NonlinearityDiagram diagramDataX = new NonlinearityDiagram();
        diagramDataX.PositiveZoneType = DiagramAfterLastStepType.ContinuousDiagramType;
        diagramDataX.PositiveZone = new double[,]
        {
          {0.0, 0.0},
          {0.5, 10000.0},
          {1.0, 15000.0},
          {3.0, 25000.0}
        };
        diagramDataX.Symmetric = false;
        diagramDataX.NegativeZoneType = DiagramAfterLastStepType.StopDiagramType;
        diagramDataX.NegativeZone = new double[,]
        {
          {0.0, 0.0},
          {-0.5, -11000.0},
          {-1.0, -12000.0},
          {-1.5, -14000.0},
          {-2.0, -18000.0},
        };

        // Creates new working diagram for the support - Z.
        NonlinearityDiagram diagramDataZ = new NonlinearityDiagram();
        diagramDataZ.PositiveZoneType = DiagramAfterLastStepType.ContinuousDiagramType;
        diagramDataZ.PositiveZone = new double[,]
        {
          {0.0, 0.0},
          {0.1, 11000.0},
          {0.2, 12000.0},
          {0.3, 13000.0},
          {0.5, 14000.0},
          {0.8, 15000.0},
          {1.3, 16000.0},
          {2.1, 17000.0}
        };
        diagramDataZ.Symmetric = true;

        // Prepares model for modification.
        data.PrepareModification();
        try
        {
          // Sets nodes.
          data.SetNodes(nodes);

          // Sets lines.
          data.SetLine(line);

          // Sets cross-section.
          data.SetCrossSection(crsc);

          // Sets hinge.
          IMemberHinge ihinge = data.SetMemberHinge(hinge);

          // For nodal support gets nonlinearity diagram interface in direction X.
          INonlinearityDiagram diagramX = ihinge.GetNonlinearity(NonlinearityDirectionType.AlongAxisX) as INonlinearityDiagram;

          // Sets diagram data in direction X.
          diagramX.SetData(diagramDataX);

          // For nodal support gets nonlinearity diagram interface in direction Z.
          INonlinearityDiagram diagramZ = ihinge.GetNonlinearity(NonlinearityDirectionType.AlongAxisZ) as INonlinearityDiagram;

          // Sets diagram data in direction Z.
          diagramZ.SetData(diagramDataZ);

          // Sets member.
          data.SetMember(member);
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        finally
        {
          // Finishes model modification
          data.FinishModification();
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        // Releases interface to RFEM model.
        model = null;

        // Unlocks license and releases interface to RFEM application.
        if (app != null)
        {
          app.UnlockLicense();
          app = null;
        }

        // Cleans Garbage Collector and releases all cached COM interfaces.
        System.GC.Collect();
        System.GC.WaitForPendingFinalizers();
      }
    }
  }
}
