﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dlubal.RFEM5;

namespace Dlubal.Examples.NonlinearSupports
{
  public partial class MainForm : Form
  {
    public MainForm()
    {
      InitializeComponent();
    }

    private void btnSetNonlinearSupports_Click(object sender, EventArgs e)
    {
      IApplication app = null;
      IModel model = null;

      try
      {
        // Gets interface to running RFEM application.
        app = Marshal.GetActiveObject("RFEM5.Application") as IApplication;
        // Locks RFEM license
        app.LockLicense();

        // Gets interface to active RFEM model.
        model = app.GetActiveModel();

        // Gets interface to model data.
        IModelData data = model.GetModelData();

        // Creates new node.
        Node nodeData = new Node();
        nodeData.No = 1;
        nodeData.X = 1.0;
        nodeData.Y = 1.0;
        nodeData.Z = 0.0;

        // Creates new support.
        NodalSupport supportData = new NodalSupport();
        supportData.No = 1;
        supportData.NodeList = nodeData.No.ToString();
        supportData.SupportNonlinearityX = NonlinearityType.WorkingDiagramType;
        supportData.SupportNonlinearityZ = NonlinearityType.WorkingDiagramType;

        // Creates new working diagram for the support - X.
        NonlinearityDiagram diagramDataX = new NonlinearityDiagram();
        diagramDataX.PositiveZoneType = DiagramAfterLastStepType.ContinuousDiagramType;
        diagramDataX.PositiveZone = new double[,]
        {
          {0.0, 0.0},
          {0.5, 10000.0},
          {1.0, 15000.0},
          {3.0, 25000.0}
        };
        diagramDataX.Symmetric = false;
        diagramDataX.NegativeZoneType = DiagramAfterLastStepType.StopDiagramType;
        diagramDataX.NegativeZone = new double[,]
        {
          {0.0, 0.0},
          {-0.5, -11000.0},
          {-1.0, -12000.0},
          {-1.5, -14000.0},
          {-2.0, -18000.0},
        };

        // Creates new working diagram for the support - Z.
        NonlinearityDiagram diagramDataZ = new NonlinearityDiagram();
        diagramDataZ.PositiveZoneType = DiagramAfterLastStepType.ContinuousDiagramType;
        diagramDataZ.PositiveZone = new double[,]
        {
          {0.0, 0.0},
          {0.1, 11000.0},
          {0.2, 12000.0},
          {0.3, 13000.0},
          {0.5, 14000.0},
          {0.8, 15000.0},
          {1.3, 16000.0},
          {2.1, 17000.0}
        };
        diagramDataZ.Symmetric = true;

        // Prepares model for modification.
        data.PrepareModification();
        try
        {
          // Sets node.
          data.SetNode(nodeData);

          // Sets nodal support and returns it's interface.
          INodalSupport support = data.SetNodalSupport(supportData);

          // For nodal support gets nonlinearity diagram interface in direction X.
          INonlinearityDiagram diagramX = support.GetNonlinearity(NonlinearityDirectionType.AlongAxisX) as INonlinearityDiagram;

          // Sets diagram data.
          diagramX.SetData(diagramDataX);

          // For nodal support gets nonlinearity diagram interface in direction Z.
          INonlinearityDiagram diagramZ = support.GetNonlinearity(NonlinearityDirectionType.AlongAxisZ) as INonlinearityDiagram;

          // Sets diagram data.
          diagramZ.SetData(diagramDataZ);
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        finally
        {
          // Finishes model modification
          data.FinishModification();
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        // Releases interface to RFEM model.
        model = null;

        // Unlocks license and releases interface to RFEM application.
        if (app != null)
        {
          app.UnlockLicense();
          app = null;
        }

        // Cleans Garbage Collector and releases all cached COM interfaces.
        System.GC.Collect();
        System.GC.WaitForPendingFinalizers();
      }
    }
  }
}
