﻿using System;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Dlubal.RFEM5;

namespace Dlubal.Examples.SurfaceOrtho
{
  public partial class FormMain : Form
  {
    public FormMain()
    {
      InitializeComponent();
    }

    private void btnGetData_Click(object sender, EventArgs e)
    {
      IApplication app = null;
      IModel model = null;

      try
      {
        // Gets interface to running RFEM application.
        app = Marshal.GetActiveObject("RFEM5.Application") as IApplication;
        // Locks RFEM licence
        app.LockLicense();

        // Gets interface to active RFEM model.
        model = app.GetActiveModel();

        // Gets interface to model data.
        IModelData data = model.GetModelData();

        // Reads all surfaces from RFEM
        Surface[] surfaces = data.GetSurfaces();

        foreach (Surface surfaceData in surfaces)
        {
          // If surface thickness type is orthotropic matrix, reads it's data
          if (surfaceData.StiffnessType == SurfaceStiffnessType.OrthotropicStiffnessType)
          {
            ISurface surface = data.GetSurface(surfaceData.No, ItemAt.AtNo);

            IOrthotropicThickness ortho = surface.GetOrthotropicThickness();
            StiffnessMatrix matrix = ortho.GetStiffnessMatrix();

            double D11 = matrix.D11;
            double D12 = matrix.D12;
            double D13 = matrix.D13;
            double D16 = matrix.D16;
            double D17 = matrix.D17;
            double D18 = matrix.D18;
            double D22 = matrix.D22;
            double D23 = matrix.D23;
            double D27 = matrix.D27;
            double D28 = matrix.D28;
            double D33 = matrix.D33;
            double D38 = matrix.D38;
            double D44 = matrix.D44;
            double D45 = matrix.D45;
            double D55 = matrix.D55;
            double D66 = matrix.D66;
            double D67 = matrix.D67;
            double D68 = matrix.D68;
            double D77 = matrix.D77;
            double D78 = matrix.D78;
            double D88 = matrix.D88;
          }
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        // Releases interface to RFEM model.
        model = null;

        // Unlocks licence and releases interface to RFEM application.
        if (app != null)
        {
          app.UnlockLicense();
          app = null;
        }

        // Cleans Garbage Collector and releases all cached COM interfaces.
        System.GC.Collect();
        System.GC.WaitForPendingFinalizers();
      }
    }
    private void btnSetData_Click(object sender, EventArgs e)
    {
      IApplication app = null;
      IModel model = null;

      try
      {
        // Gets interface to running RFEM application.
        app = Marshal.GetActiveObject("RFEM5.Application") as IApplication;
        // Locks RFEM licence
        app.LockLicense();

        // Gets interface to active RFEM model.
        model = app.GetActiveModel();

        // Gets interface to model data.
        IModelData data = model.GetModelData();

        // Allocates array of 4 nodes and sets theirs parameters.
        Node[] nodes = new Node[4];
        nodes[0].No = 1;
        nodes[0].X = 0.0;
        nodes[0].Y = 0.0;
        nodes[0].Z = 0.0;

        nodes[1].No = 2;
        nodes[1].X = 1.0;
        nodes[1].Y = 0.0;
        nodes[1].Z = 0.0;

        nodes[2].No = 3;
        nodes[2].X = 1.0;
        nodes[2].Y = 1.0;
        nodes[2].Z = 0.0;

        nodes[3].No = 4;
        nodes[3].X = 0.0;
        nodes[3].Y = 1.0;
        nodes[3].Z = 0.0;

        // Allocates array of 4 lines and sets theirs parameters.
        Line[] lines = new Line[4];
        lines[0].No = 1;
        lines[0].Type = LineType.PolylineType;
        lines[0].NodeList = "1,2";

        lines[1].No = 2;
        lines[1].Type = LineType.PolylineType;
        lines[1].NodeList = "2,3";

        lines[2].No = 3;
        lines[2].Type = LineType.PolylineType;
        lines[2].NodeList = "3,4";

        lines[3].No = 4;
        lines[3].Type = LineType.PolylineType;
        lines[3].NodeList = "4,1";

        // Allocates surface variable and sets it's parameters.
        Surface surfaceData = new Surface();
        surfaceData.No = 1;
        surfaceData.GeometryType = SurfaceGeometryType.PlaneSurfaceType;
        surfaceData.BoundaryLineList = "1-4";
        surfaceData.MaterialNo = 1;
        surfaceData.StiffnessType = SurfaceStiffnessType.OrthotropicStiffnessType;
        surfaceData.Thickness.Type = SurfaceThicknessType.ConstantThicknessType;
        surfaceData.Thickness.Constant = 0.2;

        SurfaceStiffness stiffness = new SurfaceStiffness();
        stiffness.Thickness = 0.2;
        stiffness.Type = OrthotropyType.DefinedByStiffnessMatrix;
        stiffness.MultiplicationFactors.K = 1.0;
        stiffness.MultiplicationFactors.K = 1.0;
        stiffness.MultiplicationFactors.K33 = 1.0;
        stiffness.MultiplicationFactors.K44 = 1.0;
        stiffness.MultiplicationFactors.K55 = 1.0;
        stiffness.MultiplicationFactors.Kb = 1.0;
        stiffness.MultiplicationFactors.Ke = 1.0;
        stiffness.MultiplicationFactors.Km = 1.0;
        stiffness.MultiplicationFactors.Ks = 1.0;

        StiffnessMatrix matrix = new StiffnessMatrix();
        matrix.D11 = 11;
        matrix.D12 = 12;
        matrix.D13 = 13;
        matrix.D16 = 16;
        matrix.D17 = 17;
        matrix.D18 = 18;
        matrix.D22 = 22;
        matrix.D23 = 23;
        matrix.D27 = 27;
        matrix.D28 = 28;
        matrix.D33 = 33;
        matrix.D38 = 38;
        matrix.D44 = 44;
        matrix.D45 = 45;
        matrix.D55 = 55;
        matrix.D66 = 66;
        matrix.D67 = 67;
        matrix.D68 = 68;
        matrix.D77 = 77;
        matrix.D78 = 78;
        matrix.D88 = 88;

        // Sets all objects to model data.
        data.PrepareModification();
        try
        {
          data.SetNodes(nodes);
          data.SetLines(lines);
          ISurface surface = data.SetSurface(surfaceData);

          IOrthotropicThickness ortho = surface.GetOrthotropicThickness();
          ortho.SetData(stiffness);
          ortho.SetStiffnessMatrix(matrix);
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        finally
        {
          data.FinishModification();
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        // Releases interface to RFEM model.
        model = null;

        // Unlocks license and releases interface to RFEM application.
        if (app != null)
        {
          app.UnlockLicense();
          app = null;
        }

        // Cleans Garbage Collector and releases all cached COM interfaces.
        System.GC.Collect();
        System.GC.WaitForPendingFinalizers();
      }
    }
  }
}
