﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;
using Dlubal.RFEM5;
using Dlubal.RSTAB8;

namespace Dlubal.Examples.Tendon
{
  public partial class MainForm : Form
  {
    public readonly object Model;
    public readonly string InputParams;

    public const string ModuleName = "Tendon";

    private readonly DateTime TimeStart = DateTime.Now;

    public MainForm()
    {
      InitializeComponent();
      this.Model = null;
      this.InputParams = string.Empty;
      this.tabControl.Enabled = false;
    }
    public MainForm(object model, string parameters)
    {
      InitializeComponent();
      this.Model = model;
      this.InputParams = parameters;
      this.tabControl.Enabled = true;
      this.timer.Start();
    }

    protected int[] TextListToArray(string list)
    {
      List<int> numbers = new List<int>();

      string[] interfals = list.Split(',');
      foreach (string interval in interfals)
      {
        if (interval.Contains("-"))
        {
          string[] range = interval.Split('-');
          int start, end;
          if (int.TryParse(range[0], out start) && int.TryParse(range[1], out end))
          {
            for (int value = start; value <= end; value++)
              numbers.Add(value);
          }
        }
        else
        {
          int value;
          if (int.TryParse(interval, out value))
            numbers.Add(value);
        }
      }

      return numbers.ToArray();
    }
    public int TextToInt(TextBox control)
    {
      int result = 0;
      try
      {
        result = int.Parse(control.Text);
      }
      catch (Exception ex)
      {
        control.Focus();
        control.SelectAll();
        throw ex;
      }
      return result;
    }
    public float TextToFloat(TextBox control)
    {
      float result = 0;
      try
      {
        result = float.Parse(control.Text);
      }
      catch (Exception ex)
      {
        control.Focus();
        control.SelectAll();
        throw ex;
      }
      return result;
    }
    public double TextToDouble(TextBox control)
    {
      double result = 0;
      try
      {
        result = double.Parse(control.Text);
      }
      catch (Exception ex)
      {
        control.Focus();
        control.SelectAll();
        throw ex;
      }
      return result;
    }

    protected void CleanList()
    {
      listTendons.Items.Clear();
    }
    protected void CleanData()
    {
      textNo.Text = string.Empty;
      textID.Text = string.Empty;
      textName.Text = string.Empty;
      textComment.Text = string.Empty;
      textLine.Text = string.Empty;
      textType.Text = string.Empty;
      textLC.Text = string.Empty;
      textDiameter.Text = string.Empty;
      textArea.Text = string.Empty;
      textStrands.Text = string.Empty;
      textWires.Text = string.Empty;
      textPrestressType.Text = string.Empty;
      textPrestressIndicator.Text = string.Empty;
      textPrestressProcedureType.Text = string.Empty;
      textPrestressForce.Text = string.Empty;
      textSolverGEO.Text = string.Empty;
      checkSolverSM.Checked = false;

      textMatName.Text = string.Empty;
      textMatE.Text = string.Empty;
      textMatG.Text = string.Empty;
      textMatMu.Text = string.Empty;
      textMatRho.Text = string.Empty;
      textMatAlpha.Text = string.Empty;

      listGeometry.Items.Clear();
    }

    protected string GetPath()
    {
      if (Model is RFEM5.IModel)
      {
        RFEM5.IModel model = Model as RFEM5.IModel;
        if (model == null)
          return null;
        return model.GetPath();
      }
      else if (Model is RSTAB8.IModel)
      {
        RSTAB8.IModel model = Model as RSTAB8.IModel;
        if (model == null)
          return null;
        return model.GetPath();
      }
      else
        Debug.Assert(false);
      return null;
    }
    protected string GetTempPath()
    {
      if (Model is RFEM5.IModel)
      {
        RFEM5.IModel model = Model as RFEM5.IModel;
        if (model == null)
          return null;
        return model.GetTempFolderPath();
      }
      else if (Model is RSTAB8.IModel)
      {
        RSTAB8.IModel model = Model as RSTAB8.IModel;
        if (model == null)
          return null;
        return model.GetTempFolderPath();
      }
      else
        Debug.Assert(false);
      return null;
    }
    protected void SetModified()
    {
      if (Model is RFEM5.IModel)
      {
        RFEM5.IModel model = Model as RFEM5.IModel;
        if (model != null)
          model.SetModified();
      }
      else if (Model is RSTAB8.IModel)
      {
        RSTAB8.IModel model = Model as RSTAB8.IModel;
        if (model != null)
          model.SetModified();
      }
      else
        Debug.Assert(false);
    }

    protected void PickLine()
    {
      if (Model is RFEM5.IModel)
        PickLineRF();
      else if (Model is RSTAB8.IModel)
        PickLineRS();
      else
        Debug.Assert(false);
    }
    protected void PickLineRF()
    {
      RFEM5.IModel model = Model as RFEM5.IModel; Debug.Assert(model != null);
      if (model == null)
        return;

      RFEM5.IView view = model.GetActiveView();

      try
      {
        this.Visible = false;

        string objects = string.Empty;
        view.PickObjects(RFEM5.ToolType.SelectLines, ref objects);
        textSetLines.Text = objects;
      }
      catch (Exception ex)
      {
        throw ex;
      }
      finally
      {
        this.Visible = true;
      }
    }
    protected void PickLineRS()
    {
      RSTAB8.IModel model = Model as RSTAB8.IModel; Debug.Assert(model != null);
      if (model == null)
        return;

      RSTAB8.IView view = model.GetActiveView();

      try
      {
        this.Visible = false;

        string objects = string.Empty;
        view.PickObjects(RSTAB8.ToolType.SelectMembers, ref objects);
        textSetLines.Text = objects;
      }
      catch (Exception ex)
      {
        throw ex;
      }
      finally
      {
        this.Visible = true;
      }
    }

    protected void ReadTendon(int no)
    {
      if (Model is RFEM5.IModel)
        ReadTendonRF(no);
      else if (Model is RSTAB8.IModel)
        ReadTendonRS(no);
      else
        Debug.Assert(false);
    }
    protected void ReadTendonRF(int no)
    {
      RFEM5.IModel model = Model as RFEM5.IModel; Debug.Assert(model != null);
      if (model == null)
        return;

      RFEM5.IModelData data = model.GetModelData();

      RFEM5.ITendon tendon = data.GetTendon(no, RFEM5.ItemAt.AtNo);

      RFEM5.Tendon tendonData = tendon.GetData();
      textNo.Text = tendonData.No.ToString();
      textID.Text = tendonData.ID;
      textName.Text = tendonData.Name;
      textComment.Text = tendonData.Comment;
      textLine.Text = tendonData.LineList;
      textType.Text = tendonData.Type.ToString();
      textLC.Text = tendonData.LoadList;
      textDiameter.Text = tendonData.Diameter.ToString();
      textArea.Text = tendonData.Area.ToString();
      textStrands.Text = tendonData.StrandsCount.ToString();
      textWires.Text = tendonData.WiresCount.ToString();
      textPrestressType.Text = tendonData.Prestressing.ToString();
      textPrestressIndicator.Text = tendonData.PrestressingFrom.ToString();
      textPrestressProcedureType.Text = tendonData.PrestressingProcedure.ToString();
      textPrestressForce.Text = tendonData.PrestressingForce.ToString();
      textSolverGEO.Text = tendonData.SolverGeo;
      checkSolverSM.Checked = tendonData.SolverSM;

      RFEM5.Material material = tendon.GetMaterial();
      textMatName.Text = material.Description;
      textMatE.Text = material.ElasticityModulus.ToString();
      textMatG.Text = material.ShearModulus.ToString();
      textMatMu.Text = material.PoissonRatio.ToString();
      textMatRho.Text = material.SpecificWeight.ToString();
      textMatAlpha.Text = material.ThermalExpansion.ToString();

      RFEM5.Point3D[] points = tendon.GetGeometry();
      double[] loads = tendon.GetSolverLoad();

      if (points.Length != loads.Length)
        MessageBox.Show("Count of geometry points is different to count of loads", "Tendon", MessageBoxButtons.OK, MessageBoxIcon.Warning);

      for (int i = 0; i < points.Length; i++)
      {
        ListViewItem item = new ListViewItem();
        item.Text = points[i].X.ToString();
        item.SubItems.Add(points[i].Y.ToString());
        item.SubItems.Add(points[i].Z.ToString());
        if (i < loads.Length)
          item.SubItems.Add(loads[i].ToString());
        listGeometry.Items.Add(item);
      }
    }
    protected void ReadTendonRS(int no)
    {
      RSTAB8.IModel model = Model as RSTAB8.IModel; Debug.Assert(model != null);
      if (model == null)
        return;

      RSTAB8.IModelData data = model.GetModelData();

      RSTAB8.ITendon tendon = data.GetTendon(no, RSTAB8.ItemAt.AtNo);

      RSTAB8.Tendon tendonData = tendon.GetData();
      textNo.Text = tendonData.No.ToString();
      textID.Text = tendonData.ID;
      textName.Text = tendonData.Name;
      textComment.Text = tendonData.Comment;
      textLine.Text = tendonData.MemberList;
      textType.Text = tendonData.Type.ToString();
      textLC.Text = tendonData.LoadList;
      textDiameter.Text = tendonData.Diameter.ToString();
      textArea.Text = tendonData.Area.ToString();
      textStrands.Text = tendonData.StrandsCount.ToString();
      textWires.Text = tendonData.WiresCount.ToString();
      textPrestressType.Text = tendonData.Prestressing.ToString();
      textPrestressIndicator.Text = tendonData.PrestressingFrom.ToString();
      textPrestressProcedureType.Text = tendonData.PrestressingProcedure.ToString();
      textPrestressForce.Text = tendonData.PrestressingForce.ToString();
      textSolverGEO.Text = tendonData.SolverGeo;
      checkSolverSM.Checked = tendonData.SolverSM;

      RSTAB8.Material material = tendon.GetMaterial();
      textMatName.Text = material.Description;
      textMatE.Text = material.ElasticityModulus.ToString();
      textMatG.Text = material.ShearModulus.ToString();
      textMatMu.Text = material.PoissonRatio.ToString();
      textMatRho.Text = material.SpecificWeight.ToString();
      textMatAlpha.Text = material.ThermalExpansion.ToString();

      RSTAB8.Point3D[] points = tendon.GetGeometry();
      double[] loads = tendon.GetSolverLoad();

      if (points.Length != loads.Length)
        MessageBox.Show("Count of geometry points is different to count of loads", "Tendon", MessageBoxButtons.OK, MessageBoxIcon.Warning);

      for (int i = 0; i < points.Length; i++)
      {
        ListViewItem item = new ListViewItem();
        item.Text = points[i].X.ToString();
        item.SubItems.Add(points[i].Y.ToString());
        item.SubItems.Add(points[i].Z.ToString());
        if (i < loads.Length)
          item.SubItems.Add(loads[i].ToString());
        listGeometry.Items.Add(item);
      }
    }

    protected void ReadTendons()
    {
      if (Model is RFEM5.IModel)
        ReadTendonsRF();
      else if (Model is RSTAB8.IModel)
        ReadTendonsRS();
      else
        Debug.Assert(false);
    }
    protected void ReadTendonsRF()
    {
      RFEM5.IModel model = Model as RFEM5.IModel; Debug.Assert(model != null);
      if (model == null)
        return;

      RFEM5.IModelData data = model.GetModelData();

      RFEM5.Tendon[] tendons = data.GetTendons();
      foreach (RFEM5.Tendon tendon in tendons)
      {
        listTendons.Items.Add(tendon.No);
      }
    }
    protected void ReadTendonsRS()
    {
      RSTAB8.IModel model = Model as RSTAB8.IModel; Debug.Assert(model != null);
      if (model == null)
        return;

      RSTAB8.IModelData data = model.GetModelData();

      RSTAB8.Tendon[] tendons = data.GetTendons();
      foreach (RSTAB8.Tendon tendon in tendons)
      {
        listTendons.Items.Add(tendon.No);
      }
    }

    protected void WriteTendon()
    {
      if (Model is RFEM5.IModel)
        WriteTendonRF();
      else if (Model is RSTAB8.IModel)
        WriteTendonRS();
      else
        Debug.Assert(false);
    }
    protected void WriteTendonRF()
    {
      RFEM5.IModel model = Model as RFEM5.IModel; Debug.Assert(model != null);
      Debug.Assert(model != null);
      if (model == null)
        return;

      RFEM5.IModelData data = model.GetModelData();
      if (data == null)
        return;

      int no = TextToInt(textSetNo);
      int countStrands = TextToInt(textSetStrands);
      int countWires = TextToInt(textSetWires);
      double diameter = TextToDouble(textSetDiameter);
      double area = TextToDouble(textSetArea);
      double prestressForce = TextToDouble(textSetPrestressForce);
      double e = TextToDouble(textSetMatE);
      double g = TextToDouble(textSetMatG);
      double mu = TextToDouble(textSetMatMu);
      double rho = TextToDouble(textSetMatRho);
      double alpha = TextToDouble(textSetMatAlpha);

      List<RFEM5.Point3D> listPoints = new List<RFEM5.Point3D>();
      List<double> listLoads = new List<double>();

      int[] numbers = TextListToArray(textSetLines.Text);
      foreach (int lineNo in numbers)
      {
        RFEM5.ILine line = data.GetLine(lineNo, RFEM5.ItemAt.AtNo);
        RFEM5.Line lineData = line.GetData();

        int[] nodes = TextListToArray(lineData.NodeList);
        foreach (int nodeNo in nodes)
        {
          RFEM5.Node node = data.GetNode(nodeNo, RFEM5.ItemAt.AtNo).GetData();
          RFEM5.Point3D pt = new RFEM5.Point3D();
          pt.X = node.X + 0.5;
          pt.Y = node.Y + 0.5;
          pt.Z = node.Z;

          listPoints.Add(pt);
          listLoads.Add(1000 + listLoads.Count);
        }
      }
      RFEM5.Point3D[] points = listPoints.ToArray();
      double[] loads = listLoads.ToArray();
      listPoints = null;
      listLoads = null;

      RFEM5.Tendon tendonData = new RFEM5.Tendon();
      tendonData.No = no;
      tendonData.ID = textSetID.Text;
      tendonData.Comment = textSetComment.Text;
      tendonData.Name = textSetName.Text;
      tendonData.Type = (RFEM5.TendonType)comboType.SelectedIndex;
      tendonData.LineList = textSetLines.Text;
      tendonData.LoadList = textSetLC.Text;
      tendonData.Diameter = diameter;
      tendonData.Area = area;
      tendonData.StrandsCount = countStrands;
      tendonData.WiresCount = countWires;
      tendonData.Prestressing = (RFEM5.TendonPrestressType)comboPrestressType.SelectedIndex;
      tendonData.PrestressingFrom = (RFEM5.TendonPrestressIndicatorType)comboPrestressIndicator.SelectedIndex;
      tendonData.PrestressingProcedure = (RFEM5.TendonPrestressProcedureType)comboPrestresssProcedureType.SelectedIndex;
      tendonData.PrestressingForce = prestressForce;
      tendonData.SolverGeo = textSetSolverGEO.Text;
      tendonData.SolverSM = checkSolverSM.Checked;

      RFEM5.Material material = new RFEM5.Material();
      material.Description = textSetMatName.Text;
      material.ThermalExpansion = alpha;
      material.ElasticityModulus = e;
      material.ShearModulus = g;
      material.PartialSafetyFactor = 0.0f;
      material.PoissonRatio = mu;
      material.SpecificWeight = rho;

      try
      {
        data.PrepareModification();
        RFEM5.ITendon tendon = data.SetTendon(ref tendonData);
        if (tendon != null)
        {
          tendon.SetMaterial(material);
          tendon.SetGeometry(points);
          tendon.SetSolverLoad(loads);
        }
      }
      catch (Exception ex)
      {
        throw ex;
      }
      finally
      {
        data.FinishModification();
      }
    }
    protected void WriteTendonRS()
    {
      RSTAB8.IModel model = Model as RSTAB8.IModel; Debug.Assert(model != null);
      Debug.Assert(model != null);
      if (model == null)
        return;

      RSTAB8.IModelData data = model.GetModelData();
      if (data == null)
        return;

      int no = TextToInt(textSetNo);
      int countStrands = TextToInt(textSetStrands);
      int countWires = TextToInt(textSetWires);
      double diameter = TextToDouble(textSetDiameter);
      double area = TextToDouble(textSetArea);
      double prestressForce = TextToDouble(textSetPrestressForce);
      double e = TextToDouble(textSetMatE);
      double g = TextToDouble(textSetMatG);
      double mu = TextToDouble(textSetMatMu);
      double rho = TextToDouble(textSetMatRho);
      double alpha = TextToDouble(textSetMatAlpha);

      List<RSTAB8.Point3D> listPoints = new List<RSTAB8.Point3D>();
      List<double> listLoads = new List<double>();

      int[] numbers = TextListToArray(textSetLines.Text);
      foreach (int memberNo in numbers)
      {
        RSTAB8.IMember member = data.GetMember(memberNo, RSTAB8.ItemAt.AtNo);
        RSTAB8.Member memberData = member.GetData();

        RSTAB8.Node node;
        RSTAB8.Point3D pt;

        node = data.GetNode(memberData.StartNodeNo, RSTAB8.ItemAt.AtNo).GetData();
        pt = new RSTAB8.Point3D();
        pt.X = node.X + 0.5;
        pt.Y = node.Y + 0.5;
        pt.Z = node.Z;

        listPoints.Add(pt);
        listLoads.Add(1000 + listLoads.Count);

        node = data.GetNode(memberData.StartNodeNo, RSTAB8.ItemAt.AtNo).GetData();
        pt = new RSTAB8.Point3D();
        pt.X = node.X + 0.5;
        pt.Y = node.Y + 0.5;
        pt.Z = node.Z;

        listPoints.Add(pt);
        listLoads.Add(1000 + listLoads.Count);
      }
      RSTAB8.Point3D[] points = listPoints.ToArray();
      double[] loads = listLoads.ToArray();
      listPoints = null;
      listLoads = null;

      RSTAB8.Tendon tendonData = new RSTAB8.Tendon();
      tendonData.No = no;
      tendonData.ID = textSetID.Text;
      tendonData.Comment = textSetComment.Text;
      tendonData.Name = textSetName.Text;
      tendonData.Type = (RSTAB8.TendonType)comboType.SelectedIndex;
      tendonData.MemberList = textSetLines.Text;
      tendonData.LoadList = textSetLC.Text;
      tendonData.Diameter = diameter;
      tendonData.Area = area;
      tendonData.StrandsCount = countStrands;
      tendonData.WiresCount = countWires;
      tendonData.Prestressing = (RSTAB8.TendonPrestressType)comboPrestressType.SelectedIndex;
      tendonData.PrestressingFrom = (RSTAB8.TendonPrestressIndicatorType)comboPrestressIndicator.SelectedIndex;
      tendonData.PrestressingProcedure = (RSTAB8.TendonPrestressProcedureType)comboPrestresssProcedureType.SelectedIndex;
      tendonData.PrestressingForce = prestressForce;
      tendonData.SolverGeo = textSetSolverGEO.Text;
      tendonData.SolverSM = checkSolverSM.Checked;

      RSTAB8.Material material = new RSTAB8.Material();
      material.Description = textSetMatName.Text;
      material.ThermalExpansion = alpha;
      material.ElasticityModulus = e;
      material.ShearModulus = g;
      material.PartialSafetyFactor = 0.0f;
      material.PoissonRatio = mu;
      material.SpecificWeight = rho;

      try
      {
        data.PrepareModification();
        RSTAB8.ITendon tendon = data.SetTendon(ref tendonData);
        if (tendon != null)
        {
          tendon.SetMaterial(material);
          tendon.SetGeometry(points);
          tendon.SetSolverLoad(loads);
        }
      }
      catch (Exception ex)
      {
        throw ex;
      }
      finally
      {
        data.FinishModification();
      }
    }

    protected override void OnLoad(EventArgs e)
    {
      base.OnLoad(e);

      CleanList();

      string[] groups = InputParams.Split('|');
      foreach (string group in groups)
      {
        string[] parts = group.Split(':');
        if (parts.Length < 2)
          continue;

        if (parts[0] == "Tendons")
        {
          int[] numbers = TextListToArray(parts[1]);
          foreach (int no in numbers)
          {
            listTendons.Items.Add(no);
          }
        }
      }

      if (listTendons.Items.Count > 0)
      {
        ReadTendon((int)listTendons.Items[0]);
      }
    }

    private void listTendons_SelectedIndexChanged(object sender, EventArgs e)
    {
      try
      {
        CleanData();
        int no = (int)listTendons.SelectedItem;
        ReadTendon(no);
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }
    private void btnGetAll_Click(object sender, EventArgs e)
    {
      try
      {
        CleanList();
        CleanData();
        ReadTendons();
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }
    private void btnSetTendon_Click(object sender, EventArgs e)
    {
      try
      {
        WriteTendon();
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }
    private void btnPickLine_Click(object sender, EventArgs e)
    {
      try
      {
        PickLine();
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }
    private void btnSave_Click(object sender, EventArgs e)
    {
      try
      {
        string path = GetTempPath() + "\\" + ModuleName + "\\";
        if (string.IsNullOrEmpty(path) || !Directory.Exists(path))
        {
          MessageBox.Show("Temp path does not exist", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
          return;
        }

        File.WriteAllText(path + "file1.txt", "File1 : 1,2,3,4,5,6,7");

        System.Drawing.Image img = Resource.ResourceManager.GetObject("test") as System.Drawing.Bitmap;
        img.Save(path + "file2.bmp", System.Drawing.Imaging.ImageFormat.Bmp);
        
        SetModified();
      }
      catch(Exception ex)
      {
        MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }
    private void timer_Tick(object sender, EventArgs e)
    {
      textCounter.Text = (DateTime.Now - TimeStart).ToString();

      try
      {
        RFEM5.IModel model = Model as RFEM5.IModel;
        model.GetID();
      }
      catch (Exception ex)
      {
        timer.Stop();
        MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }
  }
}
