﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dlubal.RSTAB8;

namespace Dlubal.Examples.MemberLoads
{
  public partial class FormMain : Form
  {
    public FormMain()
    {
      InitializeComponent();
    }

    private void btnSetMembers_Click(object sender, EventArgs e)
    {
      IApplication app = null;
      IModel model = null;

      try
      {
        // Gets interface to running RSTAB application.
        app = Marshal.GetActiveObject("RSTAB8.Application") as IApplication;
        // Locks RSTAB licence
        app.LockLicense();

        // Gets interface to active RSTAB model.
        model = app.GetActiveModel();

        #region Model data

        // Gets interface to model data.
        IModelData data = model.GetModelData();

        // Allocates array of 4 nodes and sets theirs parameters.
        Node[] nodes = new Node[4];
        nodes[0].No = 1;
        nodes[0].X = 0;
        nodes[0].Y = 0;
        nodes[0].Z = 0;

        nodes[1].No = 2;
        nodes[1].X = 3;
        nodes[1].Y = 0;
        nodes[1].Z = 0;

        nodes[2].No = 3;
        nodes[2].X = 3;
        nodes[2].Y = 2;
        nodes[2].Z = 0;

        nodes[3].No = 4;
        nodes[3].X = 0;
        nodes[3].Y = 2;
        nodes[3].Z = 0;

        // Allocates cross-section variable and sets it's parameters.
        CrossSection crsc = new CrossSection();
        crsc.No = 1;
        crsc.MaterialNo = 1;
        crsc.Description = "IPE 100";

        // Allocates array of 4 members and sets theirs parameters.
        Member[] members = new Member[4];
        members[0].No = 1;
        members[0].StartCrossSectionNo = 1;
        members[0].EndCrossSectionNo = 1;
        members[0].Type = MemberType.Beam;
        members[0].StartNodeNo = 1;
        members[0].EndNodeNo = 2;

        members[1].No = 2;
        members[1].StartCrossSectionNo = 1;
        members[1].EndCrossSectionNo = 1;
        members[1].Type = MemberType.Beam;
        members[1].StartNodeNo = 2;
        members[1].EndNodeNo = 3;

        members[2].No = 3;
        members[2].StartCrossSectionNo = 1;
        members[2].EndCrossSectionNo = 1;
        members[2].Type = MemberType.Beam;
        members[2].StartNodeNo = 3;
        members[2].EndNodeNo = 4;

        members[3].No = 4;
        members[3].StartCrossSectionNo = 1;
        members[3].EndCrossSectionNo = 1;
        members[3].Type = MemberType.Beam;
        members[3].StartNodeNo = 4;
        members[3].EndNodeNo = 1;

        // Nodal support parameters
        NodalSupport support = new NodalSupport();
        support.No = 0;
        support.NodeList = "1-4";
        support.SupportConstantX = -1;
        support.SupportConstantY = -1;
        support.SupportConstantZ = -1;
        support.RestraintConstantX = 0;
        support.RestraintConstantY = 0;
        support.RestraintConstantZ = -1;

        // Sets all objects to model data.
        data.PrepareModification();
        try
        {
          data.SetNodes(nodes);
          data.SetCrossSection(crsc);
          data.SetMembers(members);
          data.SetNodalSupport(support);
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        finally
        {
          data.FinishModification();
        }

        #endregion

        // Gets interface to loads
        ILoads loads = model.GetLoads();
        
        // Interface to load case
        ILoadCase lc = null;

        #region Load Case

        // Load case parameters
        LoadCase lcData = new LoadCase();
        lcData.Loading.No = 1;
        lcData.Description = "Load Case with Member Loads";
        lcData.ActionCategory = ActionCategoryType.Permanent;
        lcData.SelfWeight = false;

        loads.PrepareModification();
        try
        {
          lc = loads.SetLoadCase(lcData);
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        finally
        {
          loads.FinishModification();
        }

        #endregion

        #region Member Loads

        // Allocates array of 4 member loads and sets theris parameters.
        MemberLoad[] memberLoads = new MemberLoad[4];
        memberLoads[0].No = 0;
        memberLoads[0].ObjectList = "1";
        memberLoads[0].Type = LoadType.ForceType;
        memberLoads[0].ReferenceTo = MemberLoadReferenceType.MembersType;
        memberLoads[0].ReferenceLength = LoadReferenceLengthType.TrueMemberLengthType;
        memberLoads[0].Direction = LoadDirectionType.GlobalZType;
        memberLoads[0].Distribution = LoadDistributionType.TrapezoidalType;
        memberLoads[0].RelativeDistances = true;
        memberLoads[0].DistanceA = 0.0;
        memberLoads[0].DistanceB = 100.0;
        memberLoads[0].Magnitude1 = 10000.0;
        memberLoads[0].Magnitude2 = 20000.0;

        memberLoads[1].No = 0;
        memberLoads[1].ObjectList = "3";
        memberLoads[1].Type = LoadType.ForceType;
        memberLoads[1].ReferenceTo = MemberLoadReferenceType.MembersType;
        memberLoads[1].ReferenceLength = LoadReferenceLengthType.TrueMemberLengthType;
        memberLoads[1].Direction = LoadDirectionType.GlobalZType;
        memberLoads[1].Distribution = LoadDistributionType.TrapezoidalType;
        memberLoads[1].RelativeDistances = true;
        memberLoads[1].DistanceA = 0.0;
        memberLoads[1].DistanceB = 100.0;
        memberLoads[1].Magnitude1 = 20000.0;
        memberLoads[1].Magnitude2 = 10000.0;

        memberLoads[2].No = 0;
        memberLoads[2].ObjectList = "2";
        memberLoads[2].Type = LoadType.ForceType;
        memberLoads[2].ReferenceTo = MemberLoadReferenceType.MembersType;
        memberLoads[2].ReferenceLength = LoadReferenceLengthType.TrueMemberLengthType;
        memberLoads[2].Direction = LoadDirectionType.GlobalZType;
        memberLoads[2].Distribution = LoadDistributionType.UniformType;
        memberLoads[2].Magnitude1 = 20000.0;

        memberLoads[3].No = 0;
        memberLoads[3].ObjectList = "4";
        memberLoads[3].Type = LoadType.ForceType;
        memberLoads[3].ReferenceTo = MemberLoadReferenceType.MembersType;
        memberLoads[3].ReferenceLength = LoadReferenceLengthType.TrueMemberLengthType;
        memberLoads[3].Direction = LoadDirectionType.GlobalZType;
        memberLoads[3].Distribution = LoadDistributionType.UniformType;
        memberLoads[3].Magnitude1 = 10000.0;

        // Sets all member loads.
        lc.PrepareModification();
        try
        {
          lc.SetMemberLoads(memberLoads);
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        finally
        {
          lc.FinishModification();
        }

        #endregion
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        // Releases interface to RSTAB model.
        if (model != null)
        {
          model = null;
        }

        // Unlocks licence and releases interface to RSTAB application.
        if (app != null)
        {
          app.UnlockLicense();
          app = null;
        }

        // Cleans Garbage Collector and releases all cached COM interfaces and objects.
        System.GC.Collect();
        System.GC.WaitForPendingFinalizers();
      }
    }
  }
}
